"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Plot = Plot;
const jsx_runtime_1 = require("react/jsx-runtime");
const d3_scale_1 = require("d3-scale");
const d3_scale_chromatic_1 = require("d3-scale-chromatic");
const immer_1 = require("immer");
const react_1 = require("react");
const react_d3_utils_1 = require("react-d3-utils");
const legendContext_1 = require("../contexts/legendContext");
const plotContext_1 = require("../contexts/plotContext");
const plotControllerContext_1 = require("../contexts/plotController/plotControllerContext");
const utils_1 = require("../utils");
const splitChildren_1 = require("../utils/splitChildren");
const usePlotSizes_1 = require("../utils/usePlotSizes");
const Tracking_1 = __importDefault(require("./Tracking"));
const AnnotationsTrackingLayer_1 = __importDefault(require("./layers/AnnotationsTrackingLayer"));
const MainLayer_1 = __importDefault(require("./layers/MainLayer"));
const reducerCurr = (0, immer_1.produce)(plotContext_1.plotReducer);
const initialState = {
    headingPosition: null,
    legendPosition: null,
    legendMargin: 0,
    series: [],
    axes: {},
};
const defaultSvgStyle = {
    overflow: 'visible',
    fontFamily: 'Arial, Helvetica, sans-serif',
    userSelect: 'none',
    // Without this, Safari on iOS still triggers selection from a long press.
    WebkitUserSelect: 'none',
};
const defaultEventSvgStyle = {
    // Prevent default touch behavior on the plot only when events are enabled.
    touchAction: 'none',
};
function Plot(props) {
    const { width, height, colorScheme = d3_scale_chromatic_1.schemeSet1, margin = {}, svgStyle = {}, svgId, svgClassName, plotViewportStyle = {}, seriesViewportStyle = {}, controllerId, children, } = props;
    const plotId = (0, utils_1.useId)(undefined, 'plot');
    const [state, dispatch] = (0, react_1.useReducer)(reducerCurr, initialState, undefined);
    const { series, annotations, topAxis, rightAxis, bottomAxis, leftAxis, heading, legend, } = (0, splitChildren_1.splitChildren)(children);
    if (width === undefined) {
        throw new Error('width is mandatory');
    }
    if (height === undefined) {
        throw new Error('height is mandatory');
    }
    const plotEvents = (0, plotControllerContext_1.usePlotEventsPlotContext)({ controllerId });
    (0, react_1.useEffect)(() => {
        if (!plotEvents)
            return;
        plotEvents.registerPlot(plotId);
        return () => plotEvents.unregisterPlot(plotId);
    }, [plotEvents, plotId]);
    const plotOverrides = (0, plotControllerContext_1.usePlotOverrides)({ controllerId });
    // Bounding boxes used to adapt viewport size.
    const headingBbox = (0, react_d3_utils_1.useBBoxObserver)();
    const topAxisBbox = (0, react_d3_utils_1.useBBoxObserver)();
    const rightAxisBbox = (0, react_d3_utils_1.useBBoxObserver)();
    const bottomAxisBbox = (0, react_d3_utils_1.useBBoxObserver)();
    const leftAxisBbox = (0, react_d3_utils_1.useBBoxObserver)();
    const legendBbox = (0, react_d3_utils_1.useBBoxObserver)();
    // Distances in plot.
    const { plotWidth, plotHeight, topOffset, leftOffset, legendOffset } = (0, usePlotSizes_1.usePlotSizes)({
        width,
        height,
        margin,
        axes: state.axes,
        topAxisHeight: topAxisBbox.height,
        rightAxisWidth: rightAxisBbox.width,
        bottomAxisHeight: bottomAxisBbox.height,
        leftAxisWidth: leftAxisBbox.width,
        headingPosition: state.headingPosition,
        headingHeight: headingBbox.height,
        legendPosition: state.legendPosition,
        legendMargin: state.legendMargin,
        legendWidth: legendBbox.width,
        legendHeight: legendBbox.height,
    });
    // Set scales.
    const axisContext = (0, plotContext_1.useAxisContext)(state, plotOverrides.axes, {
        plotWidth,
        plotHeight,
    });
    const ids = (0, react_1.useMemo)(() => state.series.map(({ id }) => id), [state.series]);
    const colorScaler = (0, react_1.useMemo)(() => (0, d3_scale_1.scaleOrdinal)().range(colorScheme).domain(ids), [colorScheme, ids]);
    const plotContextValue = (0, react_1.useMemo)(() => {
        return {
            width,
            height,
            plotWidth,
            plotHeight,
            axisContext,
            colorScaler,
        };
    }, [width, height, plotWidth, plotHeight, axisContext, colorScaler]);
    const finalSvgStyle = {
        ...defaultSvgStyle,
        ...(plotEvents ? defaultEventSvgStyle : null),
        ...svgStyle,
        position: 'absolute',
        top: '0',
        left: '0',
    };
    return ((0, jsx_runtime_1.jsx)(plotContext_1.plotContext.Provider, { value: plotContextValue, children: (0, jsx_runtime_1.jsx)(plotContext_1.plotDispatchContext.Provider, { value: dispatch, children: (0, jsx_runtime_1.jsx)(legendContext_1.LegendProvider, { children: (0, jsx_runtime_1.jsxs)("div", { style: { position: 'relative', width, height }, children: [(0, jsx_runtime_1.jsx)(MainLayer_1.default, { width: width, height: height, svgStyle: finalSvgStyle, svgId: svgId, svgClassName: svgClassName, plotViewportStyle: plotViewportStyle, seriesViewportStyle: seriesViewportStyle, topOffset: topOffset, leftOffset: leftOffset, plotWidth: plotWidth, plotHeight: plotHeight, plotId: plotId, series: series, topAxisRef: topAxisBbox.ref, topAxis: topAxis, rightAxisRef: rightAxisBbox.ref, rightAxis: rightAxis, bottomAxisRef: bottomAxisBbox.ref, bottomAxis: bottomAxis, leftAxisRef: leftAxisBbox.ref, leftAxis: leftAxis, headingRef: headingBbox.ref, heading: heading }), (0, jsx_runtime_1.jsx)(AnnotationsTrackingLayer_1.default, { width: width, height: height, svgStyle: finalSvgStyle, svgId: svgId, svgClassName: svgClassName, topOffset: topOffset, leftOffset: leftOffset, legendOffset: legendOffset, legend: legend, legendRef: legendBbox.ref, plotId: plotId, annotations: annotations, tracking: plotEvents ? ((0, jsx_runtime_1.jsx)(Tracking_1.default, { plotId: plotId, plotEvents: plotEvents, stateSeries: state.series, axisContext: axisContext, plotWidth: plotWidth, plotHeight: plotHeight })) : null })] }) }) }) }));
}
//# sourceMappingURL=Plot.js.map