"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScatterSeries = ScatterSeries;
const jsx_runtime_1 = require("react/jsx-runtime");
const d3_array_1 = require("d3-array");
const react_1 = require("react");
const legendContext_1 = require("../../contexts/legendContext");
const plotContext_1 = require("../../contexts/plotContext");
const hooks_1 = require("../../hooks");
const utils_1 = require("../../utils");
const ErrorBars_1 = __importDefault(require("../ErrorBars"));
const Markers_1 = require("../Markers");
function ScatterSeries(props) {
    // Update plot context with data description
    const dispatch = (0, plotContext_1.usePlotDispatchContext)();
    const { colorScaler } = (0, plotContext_1.usePlotContext)();
    const [, legendDispatch] = (0, legendContext_1.useLegend)();
    const id = (0, utils_1.useId)(props.id, 'series');
    const { xAxis = 'x', yAxis = 'y', data, label, hidden, displayErrorBars = false, xShift: oldXShift = 0, yShift: oldYShift = 0, ...otherProps } = props;
    const { markerShape = 'circle', markerStyle = {}, errorBarsStyle, errorBarsCapStyle, errorBarsCapSize, } = otherProps;
    const isVisible = (0, hooks_1.useIsSeriesVisible)(id);
    const { xShift, yShift } = (0, hooks_1.useShift)({
        xAxis,
        yAxis,
        xShift: oldXShift,
        yShift: oldYShift,
    });
    const transform = `translate(${xShift},${yShift})`;
    (0, react_1.useEffect)(() => {
        if (!hidden) {
            legendDispatch({
                type: 'ADD_LEGEND_LABEL',
                payload: {
                    id,
                    label,
                    colorLine: 'white',
                    shape: {
                        color: markerStyle?.fill?.toString() || colorScaler(id),
                        figure: typeof markerShape === 'string' ? markerShape : 'circle',
                        hidden: false,
                    },
                },
            });
            return () => legendDispatch({ type: 'REMOVE_LEGEND_LABEL', payload: { id } });
        }
    }, [
        colorScaler,
        hidden,
        id,
        label,
        legendDispatch,
        markerShape,
        markerStyle?.fill,
    ]);
    (0, react_1.useEffect)(() => {
        const [xMin, xMax] = (0, d3_array_1.extent)(data, (d) => d.x);
        const [yMin, yMax] = (0, d3_array_1.extent)(data, (d) => d.y);
        const x = { min: xMin, max: xMax, axisId: xAxis, shift: oldXShift };
        const y = { min: yMin, max: yMax, axisId: yAxis, shift: oldYShift };
        dispatch({ type: 'addSeries', payload: { id, x, y, label, data } });
        // Delete information on unmount
        return () => dispatch({ type: 'removeSeries', payload: { id } });
    }, [dispatch, id, data, xAxis, yAxis, label, oldXShift, oldYShift]);
    if (hidden)
        return null;
    // Render stateless plot component
    const inheritedProps = {
        data,
        xAxis,
        yAxis,
    };
    const errorBarsProps = {
        hidden: !displayErrorBars,
        style: errorBarsStyle,
        capStyle: errorBarsCapStyle,
        capSize: errorBarsCapSize,
        transform,
    };
    return isVisible ? ((0, jsx_runtime_1.jsxs)("g", { children: [(0, jsx_runtime_1.jsx)(ErrorBars_1.default, { ...inheritedProps, ...errorBarsProps }), (0, jsx_runtime_1.jsx)(ScatterSeriesRender, { ...otherProps, ...inheritedProps, id: id, transform: transform })] })) : null;
}
function ScatterSeriesRender({ id, data, xAxis, yAxis, markerShape = 'circle', markerSize = 8, markerStyle = {}, pointLabel = '', pointLabelStyle = {}, displayMarkers = true, lineStyle: pointLineStyle = {}, displayLines = false, transform, }) {
    // Get scales from context
    const { axisContext, colorScaler } = (0, plotContext_1.usePlotContext)();
    const [xScale, yScale] = (0, utils_1.validateAxis)(axisContext, xAxis, yAxis);
    // calculates the path to display
    const markers = (0, react_1.useMemo)(() => {
        if (xScale === undefined || yScale === undefined) {
            return null;
        }
        const color = colorScaler(id);
        const defaultColor = { fill: color, stroke: color };
        const markers = data.map((point, i) => {
            const style = (0, utils_1.functionalStyle)(defaultColor, markerStyle, point, i, data);
            // Show marker
            const Marker = Markers_1.markersComps[(0, utils_1.functionalShape)(markerShape, point, i, data)];
            const label = (0, utils_1.functionalLabel)(pointLabel, point, i, data);
            const labelStyle = (0, utils_1.functionalStyle)({}, pointLabelStyle, point, i, data);
            const Lines = [];
            if (displayLines) {
                const lineStyle = (0, utils_1.functionalStyle)({}, pointLineStyle, point, i, data);
                const prePoint = i > 0 ? (0, utils_1.middlePoint)(point, data[i - 1]) : undefined;
                const nextPoint = data[i + 1]
                    ? (0, utils_1.middlePoint)(point, data[i + 1])
                    : undefined;
                const PreviousLine = prePoint ? ((0, jsx_runtime_1.jsx)("line", { x1: 0, y1: 0, x2: xScale(prePoint.x) - xScale(point.x), y2: yScale(prePoint.y) - yScale(point.y), style: { stroke: style.fill, ...lineStyle } }, `markers-${i}-previous`)) : null;
                const NextLine = nextPoint ? ((0, jsx_runtime_1.jsx)("line", { x1: 0, y1: 0, x2: xScale(nextPoint.x) - xScale(point.x), y2: yScale(nextPoint.y) - yScale(point.y), style: { stroke: style.fill, ...lineStyle } }, `markers-${i}-next`)) : null;
                Lines.push(PreviousLine, NextLine);
            }
            return ((0, jsx_runtime_1.jsxs)("g", { transform: `translate(${xScale(point.x)}, ${yScale(point.y)})`, children: [Lines, displayMarkers ? ((0, jsx_runtime_1.jsx)(Marker, { size: markerSize, style: { stroke: style.fill, ...style } })) : null, label ? (0, jsx_runtime_1.jsx)("text", { style: labelStyle, children: label }) : null] }, `markers-${i}`));
        });
        return markers;
    }, [
        xScale,
        yScale,
        colorScaler,
        id,
        data,
        markerStyle,
        markerShape,
        pointLabel,
        pointLabelStyle,
        pointLineStyle,
        displayLines,
        displayMarkers,
        markerSize,
    ]);
    if (!markers)
        return null;
    return ((0, jsx_runtime_1.jsx)("g", { transform: transform, className: "markers", children: markers }));
}
//# sourceMappingURL=ScatterSeries.js.map