"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.splitChildren = splitChildren;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const Annotation_1 = require("../components/Annotations/Annotation");
const Axis_1 = require("../components/Axis/Axis");
const ParallelAxis_1 = require("../components/Axis/ParallelAxis");
const Heading_1 = require("../components/Heading");
const Legend_1 = require("../components/Legend");
const BarSeries_1 = require("../components/Series/BarSeries");
const FunctionSeries_1 = require("../components/Series/FunctionSeries");
const LineSeries_1 = require("../components/Series/LineSeries");
const RangeSeries_1 = require("../components/Series/RangeSeries");
const ScatterSeries_1 = require("../components/Series/ScatterSeries");
const Series_1 = require("../components/Series/Series");
/**
 * Validates that all the children inside Plot are supported and organizes them by kind.
 */
function splitChildren(children) {
    let topAxis = null;
    let rightAxis = null;
    let bottomAxis = null;
    let leftAxis = null;
    const parallelAxes = [];
    let heading = null;
    let legend = null;
    const series = [];
    const annotations = [];
    for (const child of react_1.Children.toArray(children)) {
        if (typeof child !== 'object' || !(0, react_1.isValidElement)(child)) {
            // eslint-disable-next-line no-console
            console.error('Invalid Plot child:', child);
            throw new Error('invalid Plot child');
        }
        else if (child.type === Series_1.Series ||
            child.type === FunctionSeries_1.FunctionSeries ||
            child.type === LineSeries_1.LineSeries ||
            child.type === ScatterSeries_1.ScatterSeries ||
            child.type === RangeSeries_1.RangeSeries ||
            child.type === BarSeries_1.BarSeries) {
            series.push(child);
        }
        else if (child.type === Annotation_1.Annotations) {
            annotations.push(child);
        }
        else if (child.type === Axis_1.Axis) {
            switch (child.props.position) {
                case 'top': {
                    if (topAxis !== null) {
                        throw new Error('Plot can only have one top axis');
                    }
                    topAxis = child;
                    break;
                }
                case 'right': {
                    if (rightAxis !== null) {
                        throw new Error('Plot can only have one right axis');
                    }
                    rightAxis = child;
                    break;
                }
                case 'bottom': {
                    if (bottomAxis !== null) {
                        throw new Error('Plot can only have one bottom axis');
                    }
                    bottomAxis = child;
                    break;
                }
                case 'left': {
                    if (leftAxis !== null) {
                        throw new Error('Plot can only have one left axis');
                    }
                    leftAxis = child;
                    break;
                }
                default:
                    throw new Error('unreachable');
            }
        }
        else if (child.type === ParallelAxis_1.ParallelAxis) {
            if (parallelAxes.length === 2) {
                throw new Error('Plot can have at most two parallel axes');
            }
            parallelAxes.push(child);
        }
        else if (child.type === Heading_1.Heading) {
            if (heading !== null) {
                throw new Error('Plot can only have one Heading element');
            }
            heading = child;
        }
        else if (child.type === Legend_1.Legend) {
            if (legend !== null) {
                throw new Error('Plot can only have one Legend element');
            }
            legend = child;
        }
        else {
            // eslint-disable-next-line no-console
            console.error('Invalid Plot child:', child);
            throw new Error('invalid plot child');
        }
    }
    if (!bottomAxis && !topAxis) {
        bottomAxis = (0, jsx_runtime_1.jsx)(Axis_1.Axis, { position: "bottom" });
    }
    if (!leftAxis && !rightAxis) {
        leftAxis = (0, jsx_runtime_1.jsx)(Axis_1.Axis, { position: "left" });
    }
    for (const parallelAxis of parallelAxes) {
        const id = parallelAxis.props.id;
        if (topAxis?.props.id === id) {
            if (bottomAxis !== null) {
                throw new Error('Plot can only have one bottom axis');
            }
            bottomAxis = parallelAxis;
        }
        if (rightAxis?.props.id === id) {
            if (leftAxis !== null) {
                throw new Error('Plot can only have one left axis');
            }
            leftAxis = parallelAxis;
        }
        if (bottomAxis?.props.id === id) {
            if (topAxis !== null) {
                throw new Error('Plot can only have one top axis');
            }
            topAxis = parallelAxis;
        }
        if (leftAxis?.props.id === id) {
            if (rightAxis !== null) {
                throw new Error('Plot can only have one right axis');
            }
            rightAxis = parallelAxis;
        }
    }
    return {
        topAxis,
        rightAxis,
        bottomAxis,
        leftAxis,
        heading,
        legend,
        series,
        annotations,
    };
}
//# sourceMappingURL=splitChildren.js.map